# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum
import os

# 3rd Party
import hvac


class Vault:
  def __init__(self, vault_address: str, vault_token: str):
    self.vault_address = vault_address 
    self.vault_token = vault_token 
    self.client = hvac.Client(url=self.vault_address, token=self.vault_token)

  @classmethod
  def deserialize_data(cls, d: dict):
    key, value = None, None
    data = d.get('data', {})
    if data and len(data) == 1:
      key, value = list(data.items())[0]

    return key, value

  def get(self, key):
    return self.deserialize_data(self.client.read(key))


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from copy import deepcopy

# 3rd Party
import azure.cosmos.documents as documents
import azure.cosmos.cosmos_client as cosmos_client
import azure.cosmos.errors as errors


DEFAULT_COLLECTION_OPTIONS = {
  'offerEnableRUPerMinuteThroughput': True,
  'offerVersion': "V2",
  'offerThroughput': 400
}


class DocDB(object):

  def __init__(self, endpoint: str, master_key: str, database_name: str, collection_name: str):
    self.endpoint = endpoint
    self.master_key = master_key
    self.database_name = database_name
    self.collection_name = collection_name
    self.client = cosmos_client.CosmosClient(self.endpoint, {'masterKey': self.master_key})

  @property 
  def database_link(self):
    return 'dbs/{}'.format(self.database_name)

  @property
  def collection_link(self):
    return '{}/colls/{}'.format(self.database_link, self.collection_name)

  @property
  def client(self):
    return self._client

  @client.setter
  def client(self, value):
    self._client = value

  @property
  def endpoint(self):
    return self._endpoint

  @endpoint.setter
  def endpoint(self, value: str):
    self._endpoint = value

  @property
  def master_key(self):
    return self._master_key

  @master_key.setter
  def master_key(self, value: str):
    self._master_key = value

  @property
  def database_name(self):
    return self._database_name

  @database_name.setter
  def database_name(self, value):
    self._database_name = value

  @property
  def collection_name(self):
    return self._collection_name

  @collection_name.setter
  def collection_name(self, value):
    self._collection_name = value

  def insert_doc(self, doc: dict):
    return self.client.CreateItem(self.collection_link, doc)

  def upsert_doc(self, doc: dict):
    return self.client.UpsertItem(self.collection_link, doc)
  
  def delete_doc(self, doc_id: str, options=None):
    if options is None:
      options = {'partitionKey': doc_id}
    doc_link = '{}/docs/{}'.format(self.collection_link, doc_id)
    return self.client.DeleteItem(doc_link, options=options)

  def get_by_id(self, doc_id: str, options=None) -> dict:
    if options is None:
      options = {'partitionKey': doc_id}
    out = None
    doc_link = '{}/docs/{}'.format(self.collection_link, doc_id)  
    response = self.client.ReadItem(doc_link, options=options)
    
    if response:
      out = response

    return out

  def all_ids(self):
    return [ii['id'] for ii in self.get_all_documents()]

  def get_all_documents(self):
    return self.client.ReadItems(self.collection_link)

  def get_databases(self):
    return list(self.client.ReadDatabases())


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import time

# Local Imports
from .timber import get_logger

# 3rd Party
from confluent_kafka import Consumer, KafkaError

DEFAULT_BATCH_SIZE = 1000
SLEEP_TIME_IN_SECONDS = .1


class KafkaListener:
  def __init__(self, 
               kafka_endpoint, 
               kafka_topic_name, 
               kafka_group_name, 
               sleep_time_in_seconds=SLEEP_TIME_IN_SECONDS):
    self.kafka_endpoint = kafka_endpoint
    self.kafka_topic_name = kafka_topic_name
    self.kafka_group_name = kafka_group_name
    self.consumer = Consumer({
      'bootstrap.servers': self.kafka_endpoint, 
      'group.id': self.kafka_group_name,
      'default.topic.config': {'auto.offset.reset': 'smallest'}
    })
    self.consumer.subscribe([self.kafka_topic_name])
    self.sleep_time_in_seconds = sleep_time_in_seconds
    self.logger = get_logger('{}-{}'.format(self.kafka_topic_name, self.kafka_group_name))

  def consume(self, batch_function, batch_size=DEFAULT_BATCH_SIZE):
    batch = []
    is_running = True
    while is_running:
      time.sleep(self.sleep_time_in_seconds)
      message = self.consumer.poll()
      if not message.error():
        msg = json.loads(message.value())
        batch.append(msg)
        if len(batch) == batch_size:
          out = batch_function(batch)
          self.logger.info('In consume function; total_number_of_messages_produced={}'.format(len(batch)))
          batch = []
      elif message.error().code() != KafkaError._PARTITION_AOF:
        self.logger.error("KafkaListener had the following problem in the consume function={}".format(message.error()))
        is_running = False

    if batch:
      out = batch_function(batch)
      self.logger.info('In consume function; total_number_of_messages_consumed={}'.format(len(batch)))

    self.consumer.close()


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
from io import BytesIO

# 3rd Party Import
import requests
import numpy as np
from PIL import Image


async def get_as_base64_of_url(url):
    out = None
    resp = requests.get(url) 
    if resp.status_code == 200:
      out = base64.b64encode(resp.content)

    return out


def convert_image_to_rgb(image):
  image = Image.open(BytesIO(base64.b64decode(image)))
  image = image.convert('RGB')
  thefile = BytesIO()
  image.save(thefile, "jpeg")
  out = thefile.getvalue()
  thefile.close()

  return out 


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from copy import deepcopy

# 3rd Party
from azure.cosmosdb.table.tableservice import TableService
from azure.cosmosdb.table.tablebatch import TableBatch
from azure.cosmosdb.table.models import Entity


class PartitionKeyNotFoundError(Exception):
  pass

class ClusteringKeyNotFoundError(Exception):
  pass


class AzureTable(object):

  def __init__(self, account_name: str, 
                     account_key: str, 
                     table_name: str, 
                     partition_key_field: str,
                     clustering_key_field: str):
    self.table = TableService(account_name=account_name, 
                              account_key=account_key)
    self.table_name = self.table_name
    self.partition_key_field = partition_key_field
    self.clustering_key_field = clustering_key_field

  @property
  def partition_key_name(self) -> str:
    return 'PartitionKey'

  @property
  def clustering_key_name(self) -> str:
    return 'RowKey'

  def get_payload(self, payload: dict):
    output = None
    item = deepcopy(payload)
    partition_key = payload.get(self.partition_key_field) 
    clustering_key = payload.get(self.clustering_key_field)
    if partition_key is None:
      raise PartitionKeyNotFoundError('payload={} does not have a partition key')
    if clustering_key is None:
      raise ClusteringKeyNotFoundError('payload={} does not have a clustering key')
      
    item.update({self.partition_key_name: partition_key, 
                 self.clustering_key_name: clustering_key })

    return item

  def create(self):
    return self.table.create_table(self.table_name)

  def insert(self, item: dict):
    return self.table.insert_entity(self.table_name, self.get_payload(item)) 

  def update(self, item: dict):
    return self.table.update_entity(self.table_name, self.get_payload(item)) 

  def upsert(self, item: dict):
    return self.table.insert_or_replace_entity(self.table_name, self.get_payload(item)) 

  def insert_batch(self, items: list):
    batch = TableBatch()
    for item in items:
      batch.insert_entity(self.get_payload(item))

    return self.table.commit_batch(self.table_name, batch) 

  def get(self, partition_key: str, clustering_key: str):
    return self.table.get_entity(self.table_name, 
                                 partition_key, 
                                 clustering_key)
                              
  def get_by_partition(self, partition_key: str) -> list:
    return self.table.query_entities(self.table_name, 
                                     filter="{} eq '{}'".format(self.partition_key_name, partition_key))


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.



from enum import Enum, unique
from typing import List


FloatVector = List[float]
IntegerVector = List[int]
StringVector = List[str]
Float2DVector = List[List[float]]


@unique
class ElasticSearchStatusCode(Enum):
  YELLOW = 'yellow'
  GREEN = 'green'
  RED = 'red'



import tensorflow as tf


# Some constants.
MIN_PROB = 1e-10
MAX_PROB = 1.0
CONV_STRIDS = [1, 1, 1, 1]
POOL_STRIDS = [1, 1, 1, 1]
PADDING = 'VALID'


class TextClassifier:
    def __init__(self, sequence_len: int, label_set_size: int, vocab_size: int, 
                 embedding_dim: int, num_filters: int, filter_sizes: list,
                 keep_prob: float, beta: float, dtype: tf.DType=tf.float32):
        self.sequence_len = sequence_len
        self.label_set_size = label_set_size
        self.vocab_size = vocab_size
        self.embedding_dim = embedding_dim
        self.num_filters = num_filters
        self.filter_sizes = filter_sizes
        self.keep_prob = keep_prob
        self.beta = beta
        self.dtype = dtype

        with tf.variable_scope('Embedding', reuse=tf.AUTO_REUSE):
            tf.get_variable('E',
                            [self.vocab_size, self.embedding_dim],
                            dtype=self.dtype)

        for i, filter_size in enumerate(self.filter_sizes):
            with tf.variable_scope('ConvPool-{}'.format(filter_size), reuse=tf.AUTO_REUSE):
                tf.get_variable('W',
                                [filter_size, self.embedding_dim, 1, self.num_filters],
                                dtype=self.dtype,
                                initializer=tf.truncated_normal_initializer(.0, .1))

                tf.get_variable('b',
                                [self.num_filters],
                                dtype=self.dtype,
                                initializer=tf.constant_initializer(.1))

        with tf.variable_scope('Proj', reuse=tf.AUTO_REUSE):
            tf.get_variable('W',
                            [len(self.filter_sizes) * self.num_filters, self.label_set_size],
                            dtype=self.dtype,
                            initializer=tf.truncated_normal_initializer(.0, .1))

            tf.get_variable('b',
                            [self.label_set_size],
                            self.dtype,
                            initializer=tf.constant_initializer(.1))

    def embed(self, sequence):
      with tf.variable_scope('Embedding', reuse=tf.AUTO_REUSE):
        embeddings = tf.nn.embedding_lookup(self.get('E'), sequence)
      return embeddings

    def conv(self, embeddings):
      embeddings = tf.expand_dims(embeddings, -1)

      features = []
      for i, filter_size in enumerate(self.filter_sizes):
        with tf.variable_scope('ConvPool-{}'.format(filter_size), reuse=tf.AUTO_REUSE):
          conved = tf.nn.conv2d(embeddings,
                                self.get('W'),
                                strides=CONV_STRIDS,
                                padding=PADDING)

          conved = tf.nn.relu(tf.nn.bias_add(conved, self.get('b')))

          pooled = tf.nn.max_pool(conved,
                                  [1, self.sequence_len - filter_size + 1, 1, 1],
                                  strides=POOL_STRIDS,
                                  padding=PADDING)

          features.append(pooled)

      return tf.squeeze(tf.concat(features, 3), axis=[1, 2])

    def proj(self, feature_vec):
      with tf.variable_scope('Proj', reuse=tf.AUTO_REUSE):
        logits = tf.matmul(feature_vec, self.get('W')) + self.get('b')
      return logits

    def get(self, variable_name: str, *args, **kwargs):
      return tf.get_variable(variable_name, *args, **kwargs)

    def predict(self, sequence, training):
      feature_vec = self.conv(self.embed(sequence))

      if training:
        feature_vec = tf.nn.dropout(feature_vec, self.keep_prob)

      logits = self.proj(feature_vec)
      probs = tf.nn.softmax(logits)
      return logits, probs

    def forward(self, sequence, target_vec, training=True):
      logits, probs = self.predict(sequence, training)

      cross_entropy = tf.nn.softmax_cross_entropy_with_logits_v2(logits=logits,
                                                                  labels=target_vec)

      if self.beta > 0:
        log_probs = tf.log(tf.clip_by_value(probs, MIN_PROB, MAX_PROB))
        entropy = -tf.reduce_sum(probs * log_probs, axis=[1])
        cross_entropy = cross_entropy - self.beta * entropy

      return probs, tf.reduce_mean(cross_entropy)



# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module contains a wrapper class to interact with AzureBlob 
"""

import os

# Local Imports
from .timber import get_logger

# 3rd Party Imports
from azure.storage.blob import BlockBlobService, ContentSettings

logger = get_logger('azure_blob_client')


class AzureBlob(object):

  def __init__(self, account_name, account_key, container_name, 
               max_connections=1,
               timeout_in_seconds=60):
    self.account_name = account_name
    self.account_key = account_key
    self.container_name = container_name
    self.blob_service = None
    self.max_connections = max_connections
    self.timeout_in_seconds = timeout_in_seconds

  @property
  def account_name(self) -> str:
    return self._account_name

  @account_name.setter
  def account_name(self, value):
    self._account_name = value

  @property
  def account_key(self) -> str:
    return self._account_key

  @account_key.setter
  def account_key(self, value): 
    self._account_key = value

  @property
  def container_name(self) -> str:
    return self._container_name

  @container_name.setter
  def container_name(self, value):
    self._container_name = value

  @property
  def blob_service(self) -> BlockBlobService:
    if self._blob_service is None:
      self._blob_service = BlockBlobService(account_name=self.account_name, account_key=self.account_key)
    return self._blob_service

  @blob_service.setter
  def blob_service(self, value) :
    self._blob_service = value

  def create_container(self):
    return self.blob_service.create_container(self.container_name)

  @property
  def blob_list(self) -> list:
    return [ii.name for ii in self.blob_service.list_blobs(self.container_name)]

  @classmethod
  def progress_callback(cls, current, total) -> None:
    logger.info('{} out of {} is done'.format(current, total))

  def download_blob_to_path(self, blob_name, path) -> dict:
    """
    Downloads a blob from `blob_name` to a given file `path`
    
    :param blob_name: The blob name of the blob 
    :param path: File path where blob is going to be downloaded to 

    :returns: A dictionary that has metadata information about the blob 
    :rtype: dict 
    """
    response = self.blob_service.get_blob_to_path(self.container_name, blob_name, path, 
                                                  max_connections=self.max_connections, 
                                                  progress_callback=self.progress_callback,
                                                  timeout=self.timeout_in_seconds)
    properties = response.properties
    out = {
      'content': response.content,
      'metadata': response.metadata,
      'name': response.name,
      'properties': {
        'blob_type': properties.blob_type,
        'content_length': properties.content_length,
        'content_type': properties.content_settings.content_type,
        'etag': properties.etag,
        'last_modified': properties.last_modified,
      },
      'snapshot': response.snapshot
    }

    return out

  def create_blob(self, blob_name, path, content_settings=None) -> dict:
    """
    Uploads a blob to `blob_name` from a file using file_path: `path` 
    
    :param blob_name: The blob name of the blob 
    :param path: File path where blob is going to be uploaded from 

    :returns: A dictionary that has metadata information about the blob 
    :rtype: dict 
    """
    response = self.blob_service.create_blob_from_path(self.container_name,
                                                       blob_name,
                                                       path,
                                                       content_settings=content_settings)

    return {
      'etag': response.etag,
      'last_modified': response.last_modified,
    }

  def create_blob_from_stream(self, blob_name, stream) -> dict:
    """
    Uploads a blob to `blob_name` from `stream`
    
    :param blob_name: The blob name of the blob 
    :param stream: Stream which is going to create the blob 

    :returns: A dictionary that has metadata information about the blob 
    :rtype: dict 
    """
    response = self.blob_service.create_blob_from_stream(self.container_name,
                                                         blob_name,
                                                         stream)
    return {
      'etag': response.etag,
      'last_modified': response.last_modified,
    }

  def create_blob_from_text(self, blob_name, text) -> dict:
    """
    Uploads a blob to `blob_name` from `stream`
    
    :param blob_name: The blob name of the blob 
    :param stream: Stream which is going to create the blob 

    :returns: A dictionary that has metadata information about the blob 
    :rtype: dict 
    """
    response = self.blob_service.create_blob_from_text(self.container_name,
                                                       blob_name,
                                                       text)
    return {
      'etag': response.etag,
      'last_modified': response.last_modified,
    }

  def delete_blob(self, blob_name):
    return self.blob_service.delete_blob(self.container_name, blob_name)


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

NOMAD_META_REGION = os.environ.get('NOMAD_META_REGION')
NOMAD_META_ENV = os.environ.get('NOMAD_META_ENV')
NOMAD_META_GIT_COMMIT = os.environ.get('NOMAD_META_GIT_COMMIT')
NOMAD_META_BUILD_TIMESTAMP = os.environ.get('NOMAD_META_BUILD_TIMESTAMP')
NOMAD_META_VERSION = os.environ.get('NOMAD_META_VERSION')
CONSUL_ADDR = os.environ.get('CONSUL_ADDR')



# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.



import os
# Local Imports
from ..data_types import FloatVector, StringVector, Float2DVector
# 3rd Party Imports
import gensim



class Word2VecModel(object):

  """
  Lightweight wrapper around word2vec model so that we can use to get vector representation of words
  """

  def __init__(self, model_file_path: str):
    self.model_file_path = model_file_path
    self.model = None

  @property
  def model_file_path(self) -> str:
    return self._model_file_path

  @model_file_path.setter
  def model_file_path(self, model_file_path: str) -> None:
    self._model_file_path = model_file_path

  @property
  def model(self):
    if self._model is None:
      self._model = gensim.models.KeyedVectors.load_word2vec_format(self.model_file_path)
    return self._model

  @model.setter
  def model(self, model: str) -> None:
    self._model = model

  def get_vector(self, word: str) -> FloatVector:
    """ Returns the vector representation of the word which has 300 dimensions based on the training data
    We then convert into a Python list
    This will be an endpoint which will provide a vector for a single text
    """
    out = []
    if word in self.model.wv:
      out = self.model.word_vec(word).tolist()

    return out

  def get_vectors(self, words: StringVector) -> Float2DVector:
    """ Returns a two dimensional vector representation for multiple words
    If word is not present in the dictionary, it is going to be an empty list
    """
    return [self.get_vector(ii) for ii in words]


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.



import logging
import sys


def get_logger(name):
  logger = logging.getLogger(name)
  stream_stdout = logging.StreamHandler(sys.stdout)
  stream_stdout.setLevel(logging.INFO)
  stream_stderr = logging.StreamHandler(sys.stderr)
  stream_stderr.setLevel(logging.ERROR)
  formatter = logging.Formatter('[baybars][%(name)s][%(asctime)s][%(levelname)s]:%(message)s')
  stream_stdout.setFormatter(formatter)
  stream_stderr.setFormatter(formatter)
  logger.addHandler(stream_stdout)
  logger.addHandler(stream_stderr)
  logger.setLevel(logging.INFO)

  return logger 


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# Local Imports
from .data_types import ElasticSearchStatusCode


# 3rd Party Imports
from elasticsearch import Elasticsearch


DEFAULT_NUMBER_OF_RESULTS_IN_BATCH = 1000


class ES(object):

  def __init__(self, 
               host: str, 
               port: int, 
               index_name: str,
               doc_type: str,
               total_number_of_results_in_batch=DEFAULT_NUMBER_OF_RESULTS_IN_BATCH):
    self.es_host = host 
    self.port = port 
    self.index_name = index_name
    self.doc_type = doc_type
    self.es = Elasticsearch(self.get_host(self.es_host, self.port))
    self.total_number_of_results_in_batch = total_number_of_results_in_batch

  @classmethod
  def get_host(cls, host: str, port: int) -> list:
    return [
      {'host': host, 'port': port},
    ]

  @property
  def es(self) -> str:
    return self._es

  @es.setter
  def es(self, es: str) -> None:
    self._es = es

  @property
  def total_number_of_results_in_batch(self) -> int:
    return self._total_number_of_results_in_batch

  @total_number_of_results_in_batch.setter
  def total_number_of_results_in_batch(self, total_number_of_results_in_batch: int) -> None:
    self._total_number_of_results_in_batch = total_number_of_results_in_batch

  @property
  def es_host(self) -> str:
    return self._es_host

  @es_host.setter
  def es_host(self, es_host: str) -> None:
    self._es_host = es_host

  @property
  def index_name(self):
    return self._index_name

  @index_name.setter
  def index_name(self, index: str) -> None:
    self._index_name = index

  @property
  def port(self):
    return self._port

  @port.setter
  def port(self, new_port: int) -> int:
    self._port = new_port

  @property
  def total_number_of_results(self):
    return self._total_number_of_results_in_batch

  @total_number_of_results.setter
  def total_number_of_results(self, new_value: int) -> None:
    self._total_number_of_results_in_batch = new_value

  def get(self, id_: str) -> dict:
    return self.es.get(self.index_name, self.doc_type, id_)

  def search(body):
    """
    For a given index name and body of the query, this function will return the matching documents
    :param body: the query that is going to be executed against

    """
    return es.search(index=self.index_name,
                     body={"query": {"match_all": {}}})

  def cluster_health(self):
    return self.es.cluster.health()

  def get_status(self):
    return self.cluster_health().get('status')

  def is_green(self):
    return self.get_status() == ElasticSearchStatusCode.GREEN.value

  def is_red(self):
    return self.get_status() == ElasticSearchStatusCode.RED.value


class CategorySearch(ES):

  def __init__(self, 
               host: str, 
               port: int, 
               index_name: str, 
               doc_type: str,
               category_id: int, 
               total_number_of_results=DEFAULT_NUMBER_OF_RESULTS_IN_BATCH):
    super().__init__(host, port, index_name, doc_type, total_number_of_results)
    self.category_id = category_id

  @property
  def category_id(self) -> int:
    return self._category_id

  @category_id.setter
  def category_id(self, category_id):
    self._category_id = category_id

  def do(self, scroll='2m', search_type='scan', track_scores=True):
    search_query = 'category_id: {}'.format(self.category_id)
    page = self.es.search(q=search_query,
                          scroll=scroll,
                          size=self.total_number_of_results_in_batch,
                          track_scores=track_scores)

    sid = page['_scroll_id']
    scroll_size = page['hits']['total']
    yield page

    while scroll_size > 0:
      page = self.es.scroll(scroll_id = sid, scroll = '2m')
      if page['hits']['hits']:
        yield page
      sid = page['_scroll_id']
      scroll_size = len(page['hits']['hits'])

  @classmethod
  def get_results_in_a_list(cls, ls: list) -> list:
    out = []
    for item in ls:
      hits = item.get('hits', {}).get('hits', [])
      if hits:
        out.extend(hits)

    return out

    
class KeyValueLookup(ES):

  def __init__(self, host: str, port: int, index_name: str, doc_type: str):
    super().__init__(host, port, index_name, doc_type)




# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# 3rd Party Imports
from pyhive import presto


class Presto(object):

  def __init__(self, host: str, port: str, username: str, catalog: str):
    self.host = host
    self.port = port 
    self.username = username
    self.catalog = catalog
    self.connection = None

  @property
  def host(self) -> str:
    return self._host

  @host.setter
  def host(self, value):
    self._host = value

  @property
  def port(self) -> str:
    return self._port

  @port.setter
  def port(self, value):
    self._port = value

  @property
  def username(self) -> str:
    return self._username

  @username.setter
  def username(self, value):
    self._username = value

  @property
  def catalog(self) -> str:
    return self._catalog

  @catalog.setter
  def catalog(self, value):
    self._catalog = value

  @property
  def connection(self):
    if self._connection is None:
      self.connection = presto.connect(host=self.host, 
                                       port=self.port,
                                       username=self.username,
                                       catalog=self.catalog) 

    return self._connection

  @connection.setter
  def connection(self, value):
    self._connection = value 


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

# Local
from .timber import get_logger
# 3rd Party
from confluent_kafka import Producer

DEFAULT_TIMEOUT_IN_MILLISECONDS = 100


def value_serializer(payload: dict):
  return json.dumps(payload).encode('utf-8')


class KafkaPublisher:
  def __init__(self, 
               kafka_endpoint, 
               kafka_topic_name, 
               serializer=value_serializer, 
               timeout_ms=DEFAULT_TIMEOUT_IN_MILLISECONDS):
    self.kafka_endpoint = kafka_endpoint
    self.kafka_topic_name = kafka_topic_name
    bootstrap_servers = {'bootstrap.servers': self.kafka_endpoint}
    self.producer = Producer(bootstrap_servers)
    self.serializer = serializer
    self.logger = get_logger('{}'.format(self.kafka_topic_name))

  def publish(self, message):
    self.producer.produce(self.kafka_topic_name, 
                          self.serializer(message))
    out = self.producer.flush()
    return out

  def publish_batch(self, messages):
    out = [self.publish(message) for message in messages]
    self.producer.flush()
    return out



# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
This module has a very light wrapper to interact Azure Queue
"""

import os
# 3rd Party Imports
from azure.storage.queue import QueueService, QueueMessage


BATCH_NUMBER = 16
TIMEOUT_IN_SECONDS = 300


class AzureQueue(object):

  def __init__(self, account_name, account_key, queue_name):
    self.queue_name = queue_name
    self.queue_service = QueueService(account_name=account_name, account_key=account_key)
    self.queue_service.create_queue(self.queue_name)

  def put_message_into_queue(self, content) -> QueueMessage:
    """
    Publishes a message with `content`
    
    :param content: The queue message 

    :returns: A QueueMessage that has the message as well as metadata 
    :rtype: QueueMessage 
    """
    return self.queue_service.put_message(self.queue_name, content)

  def get_messages(self) -> list:
    """
    Retrieves all of the messages that have been published into queue 
    
    :param content: The queue message 

    :returns: List of Queue messages
    :rtype: list 
    """

    return self.queue_service.get_messages(self.queue_name)

  def delete_message_from_queue(self, message_id, pop_receipt):
    self.queue_service.delete_message(self.queue_name, message_id, pop_receipt)

  def get_message_count(self):
    queue_metadata = self.queue_service.get_queue_metadata(self.queue_name)
    return queue_metadata.approximate_message_count

  def delete(self):
    return self.queue_service.delete_queue(self.queue_name)

  def empty(self):
    messages = queue_service.get_messages(self.queue_name, 
                                          num_messages=BATCH_NUMBER, 
                                          visibility_timeout=TIMEOUT_IN_SECONDS) 
    for message in messages:
      self.queue_service.delete_message(self.queue_name, message.id, message.pop_receipt)


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time


class timer(object):
  def __init__(self, name, logger):
    self.name = name
    self.logger = logger
    self.duration = None

  def __enter__(self):
    self.start = time.time()

    return self

  def __exit__(self,ty,val,tb):
    end = time.time()
    self.duration = 1000 * (end - self.start)
    self.logger.info("{} : {} milliseconds".format(self.name, self.duration))

  @property
  def duration(self):
    return self._duration

  @duration.setter
  def duration(self, value):
    self._duration = value



# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from collections import Mapping
from enum import Enum
from types import MappingProxyType
from typing import List


FloatVector = List[float]
IntegerVector = List[int]
StringVector = List[str]
Float2DVector = List[List[float]]


class DistanceMetric(str, Enum):
  CityBlock = 'cityblock'
  Cosine = 'cosine'
  Euclidean = 'euclidean'
  L1 = 'l1'
  L2 = 'l2'
  Manhattan = 'manhattan'
  BrayCurtis = 'braycurtis'
  Canberra = 'canberra'
  Chebyshev = 'chebyshev'
  Correlation = 'correlation'
  Dice = 'dice'
  Hamming = 'hamming'
  Jaccard = 'jaccard'
  Kulsinki = 'kulsinki'
  Mahalanobis = 'mahalanobis'
  Minkowski = 'minkowski'
  Rogerstanimoto = 'rogerstanimoto'
  Russellrao = 'russellrao'
  Seuclidean = 'seuclidean'
  Sokalmichener = 'sokalmichener'
  Sokalsneath = 'sokalsneath'
  SqEuclidean = 'sqeuclidean'
  Yule = 'yule'


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from enum import Enum


class Fields(str, Enum):
  IMAGE_LINK = 'image_link'
  PRODUCT_ID = 'id'
  PRODUCT_TYPE = 'product_type' 



# 3rd Party
from baybars.timber import get_logger
import numpy as np
import tensorflow as tf 

LABEL_MAP = {
  0: 'T-shirt/top',
  1: 'Trouser',
  2: 'Pullover',
  3: 'Dress',
  4: 'Coat',
  5: 'Sandal',
  6: 'Shirt',
  7: 'Sneaker',
  8: 'Bag',
  9: 'Ankle boot',
}


class UnsupportedModeException(Exception):
  pass


MODEL_DIR = "models/fashion_model"
tf.logging.set_verbosity(tf.logging.INFO)


class FashionMNISTCNN(object):

  def __init__(self, features, labels, mode, batch_size:int =500, num_epochs:int =100, learning_rate:float =0.01, dropout_rate:float=0.4):
    self.features = features 
    self.labels = labels
    self.mode = mode
    self.logger = get_logger(str(self.__class__))
    self.batch_size = batch_size
    self.num_epochs = num_epochs
    self.learning_rate = learning_rate
    self.dropout_rate = dropout_rate
  
  def build_network(self):
    first_convolution_layer = self.cnn_2d_layer_relu(self.input_layer)
    second_convolution_layer = self.cnn_2d_layer_relu(first_convolution_layer)
    first_max_pooling_layer = self.max_pool_2d_layer(second_convolution_layer) 
    third_convolution_layer = self.cnn_2d_layer_relu(first_max_pooling_layer)
    fourth_convolution_layer = self.cnn_2d_layer_relu(third_convolution_layer)
    second_max_pooling_layer = self.max_pool_2d_layer(fourth_convolution_layer)
    reshaped_layer = self.reshape_layer(second_max_pooling_layer)
    first_dense_layer = self.dense_layer(reshaped_layer)
    first_dropout_layer = self.dropout_layer(first_dense_layer)
    second_dense_layer = self.dense_layer(first_dropout_layer)
    second_dropout_layer = self.dropout_layer(second_dense_layer)
    out_layer = self.logit_layer(second_dropout_layer)

    return out_layer

  @property
  def batch_size(self) -> int:
    return self._batch_size

  @batch_size.setter
  def batch_size(self, value) -> None:
    self._batch_size = value
  
  @property
  def num_epochs(self) -> int:
    return self._num_epochs

  @num_epochs.setter
  def num_epochs(self, value) -> None:
    self._num_epochs = value

  @property
  def dropout_rate(self) -> int:
    return self._dropout_rate

  @dropout_rate.setter
  def dropout_rate(self, value) -> None:
    self._dropout_rate = value

  @property
  def is_training(self):
    return self.mode == tf.estimator.ModeKeys.TRAIN 

  @property
  def is_evaluate(self):
    return self.mode == tf.estimator.ModeKeys.EVAL

  @property
  def is_predict(self):
    return self.mode == tf.estimator.ModeKeys.PREDICT
  
  @property
  def one_hot_labels(self):
    return tf.one_hot(indices=tf.cast(self.labels, tf.int32), depth=10)

  def loss(self, layer):
    return tf.losses.softmax_cross_entropy(onehot_labels=self.one_hot_labels, 
                                           logits=layer) 

  @property
  def prediction_structure(self, inputs):
    return {
      "classes": tf.argmax(input=inputs, axis=1),
      "probabilities": tf.nn.softmax(inputs, name="softmax_tensor"),
    }

  def predict(self):
    out = None
    if self.is_predict:
      out_layer = self.build_network()
      out = tf.estimator.EstimatorSpec(mode=self.mode, predictions=self.prediction_structure)

    return out

  def train(self, features=None, labels=None, mode=None):
    out = None
    if self.is_training:
      out_layer = self.build_network()
      loss = self.loss(out_layer)
      optimizer = tf.train.GradientDescentOptimizer(learning_rate=self.learning_rate)
      train_op = optimizer.minimize(loss=loss,
                                    global_step=tf.train.get_global_step())
      tf.summary.scalar('loss', loss)
      measure = tf.equal(tf.argmax(out_layer, 1), 
                         tf.argmax(self.one_hot_labels, 1))
      accuracy = tf.reduce_mean(tf.cast(measure, tf.float32))
      tf.summary.scalar('accuracy', accuracy)
      out = tf.estimator.EstimatorSpec(mode=self.mode, loss=loss, train_op=train_op)
    
    return out

  def train_model(self, train_data, train_labels, eval_data, eval_labels):
    # Might be better to make these into functions to get the training data and labels
    train_fn = tf.estimator.inputs.numpy_input_fn(x={"x": train_data},
                                                  y=train_labels,
                                                  batch_size=self.batch_size,
                                                  num_epochs=None,
                                                  shuffle=True)

    evaluation_fn = tf.estimator.inputs.numpy_input_fn(x={"x": eval_data},
                                                       y=eval_labels,
                                                       num_epochs=1,
                                                       shuffle=False)
    for ii in range(self.num_epochs):
      self.estimator.train(input_fn=train_fn, steps=100)
      eval_results = self.estimator.evaluate(input_fn=evaluation_fn)
      predictions = list(self.estimator.predict(input_fn=evaluation_fn))
      self.logger.info('epoch={} eval_results={} and predictions={}'.format(ii, eval_results, predictions))

    return self.estimator

  @property
  def estimator(self):
    out = None
    if self.is_training:
      out = tf.estimator.Estimator(model_fn=self.train, model_dir=MODEL_DIR)
    elif self.is_evaluate:
      out = tf.estimator.Estimator(model_fn=self.evaluate, model_dir=MODEL_DIR)
    elif self.is_predict:
      out = tf.estimator.Estimator(model_fn=self.predict, model_dir=MODEL_DIR)
    else:
      raise UnsupportedModeException("Mode: {} is not supported for building estimation".format(self.mode))
      
    return out
  
  def evaluate(self, features=None, labels=None, mode=None):
    out = None 
    if self.is_evaluate:
      out_layer = self.build_network()
      loss = self.loss(out_layer)
      evaluation_metric = {
        "accuracy": tf.metrics.accuracy(labels=self.labels, 
                                        predictions=self.prediction_structure["classes"])
      }
      out = tf.estimator.EstimatorSpec(mode=self.mode, 
                                       loss=loss, 
                                       eval_metric_ops=evaluation_metric)

    return out

  @classmethod
  def activation_layer(cls):
    return tf.nn.relu

  @classmethod
  def dense_layer(cls, inputs):
    return tf.layers.dense(inputs=inputs, 
                           units=128, 
                           activation=cls.activation_layer())

  @property
  def input_layer(self):
    return tf.reshape(self.features, [-1, 28, 28, 1])

  @classmethod
  def cnn_2d_layer_relu(cls, inputs):
    return tf.layers.conv2d(inputs=inputs, 
                            filters=64, 
                            kernel_size=[5, 5], 
                            padding="same", 
                            activation=cls.activation_layer())
  
  @classmethod
  def max_pool_2d_layer(cls, inputs):
    return tf.layers.max_pooling2d(inputs=inputs,
                                   pool_size=[2, 2],
                                   strides=2)
  @classmethod
  def reshape_layer(cls, inputs):
    return tf.reshape(inputs, [-1, 7 * 7 * 64])

  def dropout_layer(self, inputs):
    return tf.layers.dropout(inputs=inputs, rate=self.dropout_rate, training=self.is_training)

  @classmethod
  def logit_layer(cls, inputs):
    return tf.layers.dense(inputs=inputs, units=10)

  @property
  def features(self):
    return self._features

  @features.setter
  def features(self, value):
    self._features = value 

  @property
  def labels(self):
    return self._labels

  @labels.setter
  def labels(self, value):
    self._labels = value 

  @property
  def mode(self):
    return self._mode

  @mode.setter
  def mode(self, value):
    self._mode = value 

 
def _main():
  DATA_DIR = 'data'
  from tensorflow.examples.tutorials.mnist import input_data
  from sklearn.utils import shuffle
  mnist = input_data.read_data_sets(DATA_DIR, one_hot=False, validation_size=0)
  train_data = mnist.train.images  
  print('train data is loaded')
  train_labels = np.asarray(mnist.train.labels, dtype=np.int32)
  print('train labels is loaded')
  train_data, train_labels = shuffle(train_data, train_labels)
  print('eval data is loaded')
  eval_data = mnist.test.images  
  eval_labels = np.asarray(mnist.test.labels, dtype=np.int32)
  eval_data, eval_labels = shuffle(eval_data, eval_labels)
  fashion_mnist_cnn = FashionMNISTCNN(train_data, train_labels, tf.estimator.ModeKeys.TRAIN)
  training = fashion_mnist_cnn.train_model(train_data, train_labels, eval_data, eval_labels)

  return fashion_mnist_cnn, training


if __name__ == '__main__':
  cnn, out = _main()


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.




import datetime 


DATETIME_FORMAT = '%Y%m%d%H%M%S' 


def get_timestamp_in_string_format():
  return datetime.datetime.utcnow().strftime(DATETIME_FORMAT)


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import (
  abstractmethod,
  ABCMeta,
)
from enum import Enum, unique
import os
import tarfile


@unique
class CompressionMode(str, Enum):
  NO_COMPRESSION = ''
  GZIP_COMPRESSION = 'gz'
  BZIP2_COMPRESSION = 'bz2'


class AbstractTarDirectory(object):
  """
  Abstract class is going to accept a directory path and output path and tars the directory
  with the compression mode method

  """
  __metaclass__ = ABCMeta

  def __init__(self, directory_path: str, file_path: str, mode: str) -> None:
    self.directory_path = directory_path
    self.file_path = file_path
    self.mode = mode

  @property
  def write_mode(self) -> str:
    return 'w:{}'.format(self.mode)

  @property
  def read_mode(self) -> str:
    return 'r:{}'.format(self.mode)

  @property
  def directory_path(self) -> str:
    return self._directory_path

  @directory_path.setter
  def directory_path(self, value: str) -> None:
    self._directory_path = value

  @property
  def file_path(self) -> str:
    return self._file_path

  @file_path.setter
  def file_path(self, value: str) -> None:
    self._file_path = value

  @property
  def mode(self) -> str:
    return self._mode

  @mode.setter
  def mode(self, value: str) -> None:
    self._mode = value

  @abstractmethod
  def create(self) -> None:
    with tarfile.open(self.file_path, self.write_mode) as tar:
      tar.add(self.directory_path, arcname=os.path.basename(self.directory_path))

  @abstractmethod
  def extract(self) -> None:
    with tarfile.open(self.file_path, self.read_mode) as tar:
      tar.extractall(path=self.directory_path)


class TarDirectory(AbstractTarDirectory):

  def __init__(self, directory_path: str, file_path: str) -> None:
    super().__init__(directory_path, file_path, CompressionMode.NO_COMPRESSION)


class TarGZDirectory(AbstractTarDirectory):

  def __init__(self, directory_path: str, file_path: str) -> None:
    super().__init__(directory_path, file_path, CompressionMode.GZIP_COMPRESSION)


class TarBZ2Directory(AbstractTarDirectory):

  def __init__(self, directory_path: str, file_path: str) -> None:
    super().__init__(directory_path, file_path, mode=CompressionMode.BZIP2_COMPRESSION)



from setuptools import setup, find_packages

packages = find_packages()

setup(name='baybars',
      version='0.0.12',
      setup_requires=['pbr==5.1.1'],
      copyright='Copyright 2018 Jet.com',
      url='http://pypi.org/project/baybars/',
      packages=packages,
      install_requires=[
        'python-consul==1.1.0',
        'azure-storage-blob==1.4.0',
        'azure-storage-queue==1.4.0',
        'confluent-kafka==0.11.6',
        'azure-cosmos==3.0.2',
        'pysftp==0.2.9',
        'requests==2.20.1',
        'numpy==1.15.4',
        'pandas==0.23.4',
        'python-consul==1.1.0',
        'PyHive==0.6.1',
        'elasticsearch==6.3.1'
      ],
      keywords='azure kafka blob documentdb cosmosdb queue tar',
      python_requires='>=3.5',
      zip_safe=False,
      pbr=True)



# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import (abstractmethod,ABCMeta,)
import os
import gzip


class AbstractGZDirectory(object):
  """
  Abstract class is going to accept a directory path and output path and gz the directory

  """
  __metaclass__ = ABCMeta

  def __init__(self, input_file_path: str, output_file_path: str, compress_level=0) -> None:
    self.input_file_path = input_file_path
    self.output_file_path = output_file_path
    self.compress_level = compress_level

  @property
  def input_file_path(self) -> str:
    return self._input_file_path

  @input_file_path.setter
  def input_file_path(self, value: str) -> None:
    self._input_file_path = value

  @property
  def output_file_path(self) -> str:
    return self._output_file_path

  @output_file_path.setter
  def output_file_path(self, value: str) -> None:
    self._output_file_path = value

  @property
  def read_mode(self):
    return 'rb'

  @property
  def write_mode(self):
    return 'wb'

  # Need to get the following two functions working for gz file
  @abstractmethod
  def read_from_gzip(self) -> None:
    out = []
    with gzip.open(self.input_file_path, mode=self.read_mode, compresslevel=self.compress_level) as input_file, open(self.output_file_path, self.write_mode) as output_file:
      for line in input_file:
        resp = output_file.write(line)
        out.append(resp)

    return out

  @abstractmethod
  def write_to_gzip(self) -> None:
    with gzip.open(self.output_file_path, self.write_mode) as output_file, open(self.input_file_path, self.read_mode) as input_file:
      output_file.write(input_file)


class GZDirectory(AbstractGZDirectory):
  def __init__(self, input_file_path: str, output_file_path: str) -> None:
    super().__init__(input_file_path, output_file_path)


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from datetime import datetime, timedelta
import os
import sys

# Local Imports
from .timber import get_logger

# 3rd Party
import pysftp

logger = get_logger('ftp')

CURRENT_DIRECTORY = ''


class FTP(object):

  def __init__(self, server, username, password, timeout_in_seconds: int):
    self.cnopts = pysftp.CnOpts()
    self.cnopts.hostkeys = None
    self.server = server
    self.username = username
    self.password = password
    self.timeout_in_seconds = timeout_in_seconds

  def create_directory(self, directory_name):
    out = None
    with pysftp.Connection(host=self.server, username=self.username, password=self.password, cnopts=self.cnopts) as sftp:
      with sftp.cd(CURRENT_DIRECTORY):
        out = sftp.makedirs(directory_name)

    return out

  def upload_file(self, local_path, remote_directory_name, remote_file_name):
    remote_path = '{}/{}/{}'.format(CURRENT_DIRECTORY, remote_directory_name, remote_file_name)
    with pysftp.Connection(host=self.server, username=self.username, password=self.password, cnopts=self.cnopts) as sftp:
      with sftp.cd(CURRENT_DIRECTORY):
        out = sftp.put(local_path, remote_path)
        logger.info('local_path={} is put into remote_path={}'.format(local_path, remote_path))

    return out 


# Copyright 2018 Jet.com 
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#  http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import consul


class ConsulValue(object):

  def __init__(self, base_url=None):
    self.base_url = base_url or self.CONSUL_BASE_URL
    self.consul_obj = None

  @property
  def base_url(self):
    return self._base_url

  @base_url.setter
  def base_url(self, value):
    if ':' in value:
      value, _ = value.split(':')
    self._base_url = value

  @property
  def consul_obj(self):
    if self._consul_obj is None:
      self._consul_obj = consul.Consul(self.base_url)
    return self._consul_obj

  @consul_obj.setter
  def consul_obj(self, value):
    self._consul_obj = value

  def get(self, key):
    out = None
    _, data = self.consul_obj.kv.get(key)
    if data is not None:
      out = data.get('Value').decode("utf-8")

    return out

  def put(self, key, value):
    return self.consul_obj.kv.put(key, value)

  @classmethod
  def try_get_int(cls, val):
    out = None 
    try:
      out = int(val)
    except TypeError:
      pass
      
    return out